"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = void 0;
const typescript_1 = __importDefault(require("typescript"));
const httpMethodList = ["GET", "PUT", "POST", "DELETE", "OPTIONS", "HEAD", "PATCH", "TRACE"];
const createErrorResponsesTypeAlias = (typeName, factory, errorResponseNames) => {
    if (errorResponseNames.length === 0) {
        return factory.TypeAliasDeclaration.create({
            export: true,
            name: typeName,
            type: typescript_1.default.factory.createToken(typescript_1.default.SyntaxKind.VoidKeyword),
        });
    }
    return factory.TypeAliasDeclaration.create({
        export: true,
        name: typeName,
        type: factory.UnionTypeNode.create({
            typeNodes: errorResponseNames.map(name => {
                return factory.TypeReferenceNode.create({
                    name,
                });
            }),
        }),
    });
};
const createSuccessResponseTypeAlias = (typeName, factory, successResponseNames) => {
    if (successResponseNames.length === 0) {
        return factory.TypeAliasDeclaration.create({
            export: true,
            name: typeName,
            type: typescript_1.default.factory.createToken(typescript_1.default.SyntaxKind.VoidKeyword),
        });
    }
    return factory.TypeAliasDeclaration.create({
        export: true,
        name: typeName,
        type: factory.UnionTypeNode.create({
            typeNodes: successResponseNames.map(name => {
                return factory.TypeReferenceNode.create({
                    name,
                });
            }),
        }),
    });
};
const createHttpMethod = (factory) => {
    return factory.TypeAliasDeclaration.create({
        export: true,
        name: "HttpMethod",
        type: factory.TypeNode.create({ type: "string", enum: httpMethodList }),
    });
};
const createQueryParamsDeclarations = (factory) => {
    const queryParameterDeclaration = factory.InterfaceDeclaration.create({
        export: true,
        name: "QueryParameter",
        members: [
            factory.PropertySignature.create({
                readOnly: false,
                name: "value",
                optional: false,
                type: factory.TypeNode.create({ type: "any" }),
            }),
            factory.PropertySignature.create({
                readOnly: false,
                name: "style",
                optional: true,
                type: factory.TypeNode.create({ type: "string", enum: ["form", "spaceDelimited", "pipeDelimited", "deepObject"] }),
            }),
            factory.PropertySignature.create({
                readOnly: false,
                name: "explode",
                optional: false,
                type: factory.TypeNode.create({ type: "boolean" }),
            }),
        ],
    });
    const queryParametersDeclaration = factory.InterfaceDeclaration.create({
        export: true,
        name: "QueryParameters",
        members: [
            factory.IndexSignatureDeclaration.create({
                name: "key",
                type: factory.TypeReferenceNode.create({ name: "QueryParameter" }),
            }),
        ],
    });
    return [queryParameterDeclaration, queryParametersDeclaration];
};
const createObjectLikeInterface = (factory) => {
    return factory.InterfaceDeclaration.create({
        export: true,
        name: "ObjectLike",
        members: [
            factory.IndexSignatureDeclaration.create({
                name: "key",
                type: factory.TypeNode.create({ type: "any" }),
            }),
        ],
    });
};
const createEncodingInterface = (factory) => {
    return factory.InterfaceDeclaration.create({
        export: true,
        name: "Encoding",
        members: [
            factory.PropertySignature.create({
                name: "contentType",
                readOnly: true,
                optional: true,
                type: factory.TypeReferenceNode.create({
                    name: "string",
                }),
            }),
            factory.PropertySignature.create({
                name: "headers",
                readOnly: false,
                optional: true,
                type: factory.TypeReferenceNode.create({
                    name: "Record<string, any>",
                }),
            }),
            factory.PropertySignature.create({
                name: "style",
                readOnly: true,
                optional: true,
                type: factory.TypeNode.create({ type: "string", enum: ["form", "spaceDelimited", "pipeDelimited", "deepObject"] }),
            }),
            factory.PropertySignature.create({
                name: "explode",
                readOnly: true,
                optional: true,
                type: factory.TypeReferenceNode.create({
                    name: "boolean",
                }),
            }),
            factory.PropertySignature.create({
                name: "allowReserved",
                readOnly: true,
                optional: true,
                type: factory.TypeReferenceNode.create({
                    name: "boolean",
                }),
            }),
        ],
    });
};
const create = (factory, list, methodType, option) => {
    const objectLikeOrAnyType = factory.UnionTypeNode.create({
        typeNodes: [
            factory.TypeReferenceNode.create({
                name: "ObjectLike",
            }),
            factory.TypeNode.create({
                type: "any",
            }),
        ],
    });
    const encodingInterface = createEncodingInterface(factory);
    const requestArgs = factory.ParameterDeclaration.create({
        name: "requestArgs",
        type: factory.TypeReferenceNode.create({
            name: "RequestArgs",
        }),
    });
    const options = factory.ParameterDeclaration.create({
        name: "options",
        optional: true,
        type: factory.TypeReferenceNode.create({
            name: "RequestOption",
        }),
    });
    const successResponseNames = list.map(item => item.convertedParams.responseSuccessNames).flat();
    const errorResponseNamespace = factory.Namespace.create({
        export: true,
        name: "ErrorResponse",
        statements: list.map(item => {
            return createErrorResponsesTypeAlias(`${item.convertedParams.escapedOperationId}`, factory, item.convertedParams.responseErrorNames);
        }),
    });
    const returnType = option.sync
        ? factory.TypeReferenceNode.create({
            name: "T",
        })
        : factory.TypeReferenceNode.create({
            name: "Promise",
            typeArguments: [
                factory.TypeReferenceNode.create({
                    name: "T",
                }),
            ],
        });
    const functionType = factory.FunctionTypeNode.create({
        typeParameters: [
            factory.TypeParameterDeclaration.create({
                name: "T",
                defaultType: factory.TypeReferenceNode.create({
                    name: "SuccessResponses",
                }),
            }),
        ],
        parameters: [requestArgs, options],
        type: returnType,
    });
    const requestFunction = factory.PropertySignature.create({
        readOnly: false,
        name: "request",
        optional: false,
        type: functionType,
    });
    const requestArgsInterfaceDeclaration = factory.InterfaceDeclaration.create({
        export: true,
        name: "RequestArgs",
        members: [
            factory.PropertySignature.create({
                name: `httpMethod`,
                readOnly: true,
                optional: false,
                type: factory.TypeReferenceNode.create({ name: "HttpMethod" }),
            }),
            factory.PropertySignature.create({
                name: methodType === "currying-function" ? "uri" : "url",
                readOnly: true,
                optional: false,
                type: factory.TypeReferenceNode.create({ name: "string" }),
            }),
            factory.PropertySignature.create({
                name: `headers`,
                readOnly: false,
                optional: false,
                type: objectLikeOrAnyType,
            }),
            factory.PropertySignature.create({
                name: `requestBody`,
                readOnly: false,
                optional: true,
                type: objectLikeOrAnyType,
            }),
            factory.PropertySignature.create({
                name: `requestBodyEncoding`,
                readOnly: false,
                optional: true,
                type: factory.TypeReferenceNode.create({ name: "Record<string, Encoding>" }),
            }),
            factory.PropertySignature.create({
                name: `queryParameters`,
                optional: true,
                readOnly: false,
                type: factory.UnionTypeNode.create({
                    typeNodes: [
                        factory.TypeReferenceNode.create({
                            name: "QueryParameters",
                        }),
                        factory.TypeNode.create({ type: "undefined" }),
                    ],
                }),
            }),
        ],
        typeParameters: [],
    });
    return [
        createHttpMethod(factory),
        createObjectLikeInterface(factory),
        ...createQueryParamsDeclarations(factory),
        createSuccessResponseTypeAlias("SuccessResponses", factory, successResponseNames),
        errorResponseNamespace,
        encodingInterface,
        requestArgsInterfaceDeclaration,
        factory.InterfaceDeclaration.create({
            export: true,
            name: "ApiClient",
            members: [requestFunction],
            typeParameters: [
                factory.TypeParameterDeclaration.create({
                    name: "RequestOption",
                }),
            ],
        }),
    ];
};
exports.create = create;
//# sourceMappingURL=ApiClientInterface.js.map